function [op,x,psf,o] = convAiry(subj,p,outputFlag)
    %Convolves Airy pattern with subject of period p in units of lambdaN
    %Subject can be '2 points','2 lines','points','lines','sin','sinstar','1951'
    %p = 1.2197 is Rayleigh's limit, 0.94 is Sparrow's. Normalize to taste.
    %EXAMPLE: [op,x,psf,o] = convAiry('sin',1.2197);
    %The key to getting the FTs to work properly is to put the dark part
    %of a cycle in the center and to have an integer number of cycles so
    %the edges are at zero intensity.  By Jack Hogan, v0.1 04/2024
    if nargin < 3
        outputFlag = 1; end

    n = 2048;                           %px, size of psf and subject
    cycles = 8;                         %number of cycles from center to edge
    pn = n/(cycles*2);                  %px, separation of objects in subject
    ctr = n/2+1;                        %center pixel
    
    [psf,x,X,Y] = makeAiry(n,cycles*p); %Airy pattern at the same scale
    psf = psf / sum(sum(psf));          %normalize total intesity of PSF to 1
    o = loadSubject(subj);
    
    OP = fft2(o) .* fft2(psf);          %convolution with psf
    op = fftshift(ifft2(OP));
    
    if ismember(subj,{'2 points','2p','2 lines','2l','points','1951'})
        op = op/max(max(op)); end       %normalize max intensity to taste
    
    %% Output
    if outputFlag
        Imax = max(max(op));
        Imin = op(1025,1025);
        mtf = (Imax-Imin)/(Imax+Imin);
        disp(['Intensity at central dip = ' num2str(Imin)])
        disp(['MTF = ' num2str(mtf)])
        figure
        subplot(1,2,1); imshow(lin2srgb(op))    %approximate gamma
        title(['Period of ' subj ' = ' num2str(p) '\lambda N'])
        subplot(1,2,2); plot(x(1:n),op(ctr,:))
        xlim([-cycles cycles]*p); ylim([0 1.1])
        grid
        xlabel('\lambda N'); ylabel('Relative Intensity')
        title('Central Slice')
    end
    %%
    function [psf,x,X,Y] = makeAiry(n,scale)
        %generates nxn Airy pattern, units of scale and x are lambdaN
        %to display apply appropriate gamma
        x = (-1/2:1/n:1/2-1/n)*scale*2;             %lambdaN
        [X,Y] = meshgrid(x);
        R = sqrt(X.^2+Y.^2);
        psf = ( 2 * besselj(1,pi*R) ./ (pi*R) ).^2; %intensity
        psf(n/2+1,n/2+1) = 1;
    end
    
    function o = loadSubject(subj)
        switch lower(subj)
            case '1951'
                o = imread('C:\Users\augus\Downloads\1951usaf_test_target.png');
            case {'2 points','2p'}
                o = zeros(n);
                o(ctr,floor(ctr-pn/2+0.5)) = 1;
                o(ctr,floor(ctr+pn/2+0.5)) = 1;
            case {'2 lines','2l'}
                o = zeros(n);
                pl = ctr-pn/2;
                pu = ctr+pn/2;
                o(:,floor(pl-pn/4+0.5):floor(pl+pn/4+0.5)) = 1;
                o(:,floor(pu-pn/4+0.5):floor(pu+pn/4+0.5)) = 1;
            case {'lin','lines'}
                o = zeros(n);
                for i = 0:2*cycles-1
                    o(:,round(pn/4+pn*i):round(pn/4+pn*i+pn/2)-1) = 1;
                end
            case 'points'                       %point train
                o = zeros(n);
                o(ctr,ctr + floor(0.5+pn/2) + floor(0.5+(0:cycles-1))*pn) = 1;
                o(ctr,ctr - floor(0.5+pn/2) - floor(0.5+(0:cycles-1))*pn) = 1;
            case {'sin','sine','cos'}
                o = 0.5 + 0.5 * cos( pi + 2*pi*(-n/2:n/2-1)/pn );
                o = repmat(o,[n 1]);
            case 'sinstar'                      %siemens star, sine
                %somewhat of an arbitrary scale
                spokes = 4 * cycles;
                %[X,Y] = meshgrid(-n/2:n/2-1);
                PHY = atan2(Y,X) * spokes;      %phase at each pixel
                o = 0.5 + 0.5*cos(PHY);         %relative intensity
            otherwise
                warning('Unrecognized Subject')
        end
    end
    
    function rgb = lin2srgb(rgb)
        %Applies sRGB gamma to linear rgb float data normalized to one 
        %and returns it as uint8 for display only
        lin = rgb <= 0.0031308;                         %sRGB linear pixels
        rgb(lin) = rgb(lin) * 12.92;                    %sRGB linear
        rgb(~lin) = 1.055*rgb(~lin).^(1/2.4) - 0.055;   %sRGB gamma
        rgb = uint8(rgb*(2^8-1));
    end

end